#include <iostream>
#include <iomanip>
#include <climits>
#include "ui.h"

using namespace std;

void ui_printMenu();
void ui_printComputers(const Computers& computers);
void ui_printCustomers(const Customers& customers);
void ui_addComputer(Computers& computers);
void ui_addCustomer(Customers& customers);
void ui_rentComputer(Computers& computers, Customers& customers);
void ui_returnComputer(Computers& computers);
void ui_printRentedComputers(const Computers& computers);
void ui_printCustomersInformation(const Customers& customers, Computers& computers);
void ui_MostMoneyEarnedComputer(const Computers& computers);
void ui_MostReturnedComputer(const Computers& computers);
void ui_balance(const Computers& computers, const Customers& customers);
void ui_printPopularComputer(const Computers& computers);

void ui_runWith(ComputerRental& rental) {
    ui_printMenu();
    for (;;) {
        cout << "\n----------------------------------------\n";
        cout << "(m - rodyti meniu) > ";
        string key;
        cin >> key;
        if (key == "0") break;
        else if (key == "m") ui_printMenu();
        else if (key == "1") ui_printComputers(rental.computers);
        else if (key == "2") ui_printRentedComputers(rental.computers);
        else if (key == "3") ui_addComputer(rental.computers);
        else if (key == "4") ui_rentComputer(rental.computers, rental.customers);
        else if (key == "5") ui_returnComputer(rental.computers);
        else if (key == "6") ui_printCustomers(rental.customers);
            //      else if (key == "7") ui_printCustomersInformation(rental.customers, rental.computers);
        else if (key == "8") ui_addCustomer(rental.customers);
        else if (key == "9") ui_MostMoneyEarnedComputer(rental.computers);
        else if (key == "10") ui_MostReturnedComputer(rental.computers);
        else if (key == "12") ui_balance(rental.computers, rental.customers);
        else if (key == "11") ui_printPopularComputer(rental.computers);
        else cout << endl << "Neteisinga komanda..." << endl;
    }
}

void ui_printMenu() {
    cout << endl;
    cout << "Kompiuterio nuoma:" << endl;
    cout << "  1  - Parodyti turimus kompiuterius" << endl;
    cout << "  2  - Parodyti isnuomotus ir laisvus kompiuterius" << endl;
    cout << "  3  - Prideti nauja kompiuteri" << endl;
    cout << "  4  - Isnuomuoti kompiuteri klientui" << endl;
    cout << "  5  - Kompiuteris grazintas" << endl;
    cout << endl;
    cout << "  6  - Spausdinti klientus" << endl;
    cout << "  8  - Prideti nauja klienta" << endl;
    cout << endl;
    cout << "  9  - Daugiausiai uzdirbes kompiuteris" << endl;
    cout << "  10 - Daugiausiai kartu grazintas kompiuteris" << endl;
    cout << "  11 - Populiariausias kompiuteris" << endl;
    cout << endl;
    cout << "  12 - Balnsas" << endl;
    cout << endl;
    cout << "  0  - baigti darba" << endl;
}

void ui_printComputer(const Computer& computer) {
    cout << " " << computer.rentedByCustomer;
    cout << "\t" << computer.operatingSystem;
    cout << "\t " << computer.hourPrice << "Lt";
    cout << "\t\t(" << computer.moneyEarned << "Lt)";
    cout << "\t\t" << computer.ID;
}

void ui_printRentedComputer(const Computer& computer) {
    if (computer.rentedByCustomer != 0) {
        cout << " " << computer.rentedByCustomer;
        cout << "\t" << computer.operatingSystem;
        cout << "\t " << computer.hourPrice << "Lt";
        cout << "\t\t(" << computer.moneyEarned << "Lt)";
        cout << "\t\t" << computer.ID;
    } else {
        cout << "Kompiuteris yra laisvas";
    }
}

void ui_printRentedComputers(const Computers& computers) {
    cout << "Kompiuteriu kiekis: " << computers.size() << endl;
    cout << "	OS	Kaina valandai	Uzdirbo(Lt)	Inventorizacijos Nr." << endl;
    for (int i = 0; i < computers.size(); i++) {
        cout << (i + 1) << ".";
        ui_printRentedComputer(computers[i]);
        cout << endl;
    }
}

void ui_printComputers(const Computers& computers) {
    cout << "Kompiuteriu kiekis: " << computers.size() << endl;
    cout << "	OS	Kaina valandai	Uzdirbo(Lt)	Inventorizacijos Nr." << endl;
    for (int i = 0; i < computers.size(); i++) {
        cout << (i + 1) << ".";
        ui_printComputer(computers[i]);
        cout << endl;
    }
}

void ui_printCustomer(const Customer& customer) {
    cout << " (" << customer.moneyEarned << "Lt)";
    cout << "\t" << customer.name;
    cout << " (" << customer.personalcode << ")";
}

void ui_printCustomers(const Customers& customers) {
    cout << "Klietu kiekis: " << customers.size() << endl;
    for (int i = 0; i < customers.size(); i++) {
        cout << (i + 1) << ".";
        ui_printCustomer(customers[i]);
        cout << endl;
    }
}

void ui_printCustomerRentedComputer(const Computers& computers, int customerNo) {
    for (int i = 0; i < computers.size(); i++) {
        if (customerNo == computers[i].rentedByCustomer) {
            cout << "\t" << computers[i].operatingSystem;
            cout << "\t\t" << computers[i].ID;
        }
    }
}

void ui_skipNewLineSymbol(istream & is) {
    is.ignore(INT_MAX, '\n');
}

void ui_addComputer(Computers & computers) {
    ui_skipNewLineSymbol(cin);
    string ID;
    string operatingSystem;
    double hourPrice;
    double computerPrice;
    int returnedCount;
    int rentedTimes;
    cout << "Iveskite nauja kompiuteri:" << endl;
    cout << " Prasome ivesti inventorizacijos Nr  : ";
    getline(cin, ID);
    cout << " Prasome ivesti kompiuterio operacine systema: Windows/Mac/Linux   : ";
    cin >> operatingSystem;
    cout << " Kaina valandai : ";
    cin >> hourPrice;
    cout << " Kiek kainavo kompiuterio isigijimas? :";
    cin >> computerPrice;
    returnedCount = 0;
    rentedTimes = 0;
    computers.push_back(computer_create(ID, operatingSystem, hourPrice, computerPrice, returnedCount, rentedTimes));
}

void ui_addCustomer(Customers & customers) {
    ui_skipNewLineSymbol(cin);
    string name;
    string personalcode;
    cout << "Iveskite nauja klienta:" << endl;
    cout << "    vardas pavarde : ";
    getline(cin, name);
    cout << "    asmens kodas: ";
    getline(cin, personalcode);
    customers.push_back(customer_create(name, personalcode));
}

void ui_rentComputer(Computers& computers, Customers & customers) {
    int computerNo = 0;
    ui_printComputers(computers);
    cout << "Iveskit kompiuterio eiles numeri: ";
    cin >> computerNo;
    if (!computers_isComputerNoValid(computers, computerNo)) {
        cout << "KLAIDA: nera kompiuterio su ivestu eiles numeriu.";
        return;
    }
    if (computers_isRented(computers, computerNo)) {
        cout << "KLAIDA: kompiuteris jau yra isnuomotas.";
        return;
    }
    int customerNo = 0;
    ui_printCustomers(customers);
    cout << "Pasirinkite kliento eiles numeri: ";
    cin >> customerNo;
    if (customerNo <= 0 || customerNo > customers.size()) {
        cout << "KLAIDA: nera kliento su ivestu eiles numeriu.";
        return;
    }
    int hourCount;
    cout << "Keliom valandom nuomuosime        : ";
    cin >> hourCount;
    computers[computerNo - 1].rentedByCustomer = customerNo;
    double moneyEarned = hourCount * computers[computerNo - 1].hourPrice;
    computers[computerNo - 1].moneyEarned += moneyEarned;
    customers[customerNo - 1].moneyEarned += moneyEarned;
    computers[customerNo - 1].rentedTimes++;
}

void ui_returnComputer(Computers & computers) {
    int computerNo = 0;
    ui_printComputers(computers);
    cout << "Iveskit kompiuterio eiles numeri: ";
    cin >> computerNo;
    if (computerNo <= 0 || computerNo > computers.size()) {
        cout << "KLAIDA: nera kompiuterio su ivestu eiles numeriu.";
        return;
    }
    computers[computerNo - 1].returnedCount = computers[computerNo - 1].returnedCount + 1; // Gal but cia yra grybas + dar kitos vietos kur pakeciau !!!!!!!!!!!!
    computers[computerNo - 1].rentedByCustomer = Computer::NOT_RENTED;
}

void ui_MostReturnedComputer(const Computers & computers) {
    int MostReturnedComputer = computers[0].returnedCount;
    int MostReturnedComputerNo;
    for (int i = 1; i < computers.size(); i++) {
        if (MostReturnedComputer <= computers[i].returnedCount) {
            MostReturnedComputer = computers[i].returnedCount;
        }
    }
    for (int i = 0; i < computers.size(); i++) {
        if (MostReturnedComputer == computers[i].returnedCount) {
            ui_printComputer(computers[i]);
        }
    }
    cout << endl;
}

void ui_MostMoneyEarnedComputer(const Computers & computers) {
    double MostMoneyEarnedComputer = computers[0].moneyEarned;
    double MostMoneyEarnedComputerNo;
    for (int i = 1; i < computers.size(); i++) {
        if (MostMoneyEarnedComputer <= computers[i].moneyEarned) {
            MostMoneyEarnedComputer = computers[i].moneyEarned;
            // MostMoneyEarnedComputerNo = i;
        }
    }
    for (int i = 0; i < computers.size(); i++) {
        if (MostMoneyEarnedComputer == computers[i].moneyEarned) {
            //    cout<<"dirba";
            ui_printComputer(computers[i]);
        }
    }
}/*
void ui_printCustomerInformation(const Customers& customers, Computer& computer) {
    int customerNo = 0;
    for (int i = 0; i < customers.size(); i++) {
        cout << (i + 1) << ".";
        ui_printCustomer(customers[i]);
    }
    cout << "Iveskite kliento eiles numeri : "; //========================================================================================================
    cin >> customerNo;
    if (customerNo <= 0 || customerNo > customers.size()) {
        cout << "KLAIDA: nera kliento su ivestu eiles numeriu.";
        return;
    }
    ui_printCustomerRentedComputer(computers, customerNo);
}
void ui_printCustomersInformation(const Customers& customers, Computers & computers) {
    cout << "Klientu kiekis: " << customers.size() << endl;
    for (int i = 0; i < customers.size(); i++) {
        cout << (i + 1) << ".";
        ui_printCustomerInformation(customers[i], computers[i]);
        cout << endl;
    }
}*/

void ui_balance(const Computers& computers, const Customers& customers) {
    double computerPriceSum = 0;
    for (int i = 0; i < computers.size(); i++) {
        computerPriceSum += computers[i].computerPrice;
    }
    double moneyEarnedSum = 0;
    for (int i = 0; i < customers.size(); i++) {
        moneyEarnedSum += customers[i].moneyEarned;
    }
    double profit = 0;
    profit = moneyEarnedSum - computerPriceSum;
    cout << "Islaidos :" << computerPriceSum << "(Lt)" << endl;
    cout << "Pajamos  :" << moneyEarnedSum << "(Lt)" << endl;
    cout << "Pelnas   :" << profit << "(Lt)" << endl;
    if ((profit > 0) && (profit <= 100))
        cout << "Siuo metu jus turite pelna, bet esate arti nulio ribos, bukite atidus!";
    else if (profit > 100)
        cout << "Siuo metu jus turite pelna";
    else if ((profit < 0) && (profit >= -100))
        cout << "Siuo metu jus turite nedidelius nuostoli, bukite atidus!";
    else if (profit < -100)
        cout << "Siuo metu jus turite nuostoli, laikas kazka keisti!";


}

void ui_printPopularComputer(const Computers& computers) {
    int max = 0;
    int id = 0;
    int nr = 0;
    cout << " Populiariausias kompiuteris : " << endl;
    for (int i = 0; i < computers.size(); i++) {
        Computer popular = computers[i];
        if (popular.rentedTimes > max) {
            max = popular.rentedTimes;
            id = i;
        }
    }
    for (int i = 0; i < computers.size(); i++) {
        Computer popular = computers[i];
        if (popular.rentedTimes == max) {
            nr = i;
            ui_printComputer(computers[nr]);
            cout << endl;
        }
    }
}
